@@ -514,65 +514,6 @@ totem_pl_parser_write (TotemPlParser *parser, GtkTreeModel *model,
 	return FALSE;
 }
 
-static GnomeVFSResult
-my_eel_read_entire_file (const char *uri,
-		int *file_size,
-		char **file_contents)
-{
-	GnomeVFSResult result;
-	GnomeVFSHandle *handle;
-	char *buffer;
-	GnomeVFSFileSize total_bytes_read;
-	GnomeVFSFileSize bytes_read;
-
-	*file_size = 0;
-	*file_contents = NULL;
-
-	/* Open the file. */
-	result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
-	if (result != GNOME_VFS_OK) {
-		return result;
-	}
-
-	/* Read the whole thing. */
-	buffer = NULL;
-	total_bytes_read = 0;
-	do {
-		buffer = g_realloc (buffer, total_bytes_read + READ_CHUNK_SIZE);
-		result = gnome_vfs_read (handle,
-				buffer + total_bytes_read,
-				READ_CHUNK_SIZE,
-				&bytes_read);
-		if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF) {
-			g_free (buffer);
-			gnome_vfs_close (handle);
-			return result;
-		}
-
-		/* Check for overflow. */
-		if (total_bytes_read + bytes_read < total_bytes_read) {
-			g_free (buffer);
-			gnome_vfs_close (handle);
-			return GNOME_VFS_ERROR_TOO_BIG;
-		}
-
-		total_bytes_read += bytes_read;
-	} while (result == GNOME_VFS_OK);
-
-	/* Close the file. */
-	result = gnome_vfs_close (handle);
-	if (result != GNOME_VFS_OK) {
-		g_free (buffer);
-		return result;
-	}
-
-	/* Return the file. */
-	*file_size = total_bytes_read;
-	*file_contents = g_realloc (buffer, total_bytes_read);
-
-	return GNOME_VFS_OK;
-}
-
 static int
 read_ini_line_int (char **lines, const char *key)
 {
@@ -657,19 +598,29 @@ totem_pl_parser_finalize (GObject *object)
 	}
 }
 
+static gboolean
+totem_pl_parser_check_utf8 (const char *title)
+{
+	return title ? g_utf8_validate (title, -1, NULL) : FALSE;
+}
+
 static void
 totem_pl_parser_add_one_url (TotemPlParser *parser, const char *url, const char *title)
 {
 	g_signal_emit (G_OBJECT (parser), totem_pl_parser_table_signals[ENTRY],
-		       0, url, title, NULL);
+		       0, url,
+		       totem_pl_parser_check_utf8 (title) ? title : NULL,
+		       NULL);
 }
 
 static void
-totem_pl_parser_add_one_url_ext (TotemPlParser *parser, const char *url, const char *title,
-			     const char *genre)
+totem_pl_parser_add_one_url_ext (TotemPlParser *parser, const char *url,
+		const char *title, const char *genre)
 {
 	g_signal_emit (G_OBJECT (parser), totem_pl_parser_table_signals[ENTRY],
-		       0, url, title, genre);
+		       0, url,
+		       totem_pl_parser_check_utf8 (title) ? title : NULL,
+		       genre);
 }
 
 static TotemPlParserResult
@@ -680,7 +631,7 @@ totem_pl_parser_add_ram (TotemPlParser *parser, const char *url, gpointer data)
 	int size, i;
 	const char *split_char;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	contents = g_realloc (contents, size + 1);
@@ -764,7 +715,7 @@ totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url, gpointer data)
 	const char *split_char;
 	gboolean extinfo;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return FALSE;
 
 	contents = g_realloc (contents, size + 1);
@@ -846,7 +797,7 @@ totem_pl_parser_add_asf_parser (TotemPlParser *parser, const char *url,
 	char *contents, **lines, *ref;
 	int size;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	contents = g_realloc (contents, size + 1);
@@ -884,7 +835,7 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 	char *split_char;
 	gboolean dos_mode = FALSE;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	if (size == 0)
@@ -1058,6 +1009,11 @@ parse_asx_entries (TotemPlParser *parser, char *base, xmlDocPtr doc,
 			if (parse_asx_entry (parser, base, doc, parent, title) != FALSE)
 				retval = TRUE;
 		}
+		if (g_ascii_strcasecmp (node->name, "repeat") == 0) {
+			/* Repeat at the top-level */
+			if (parse_asx_entries (parser, base, doc, node) != FALSE)
+				retval = TRUE;
+		}
 	}
 
 	g_free (title);
@@ -1074,7 +1030,7 @@ totem_pl_parser_add_asx (TotemPlParser *parser, const char *url, gpointer data)
 	int size;
 	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return FALSE;
 
 	contents = g_realloc (contents, size + 1);
@@ -1205,7 +1161,7 @@ totem_pl_parser_add_smil (TotemPlParser *parser, const char *url, gpointer data)
 	int size;
 	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	contents = g_realloc (contents, size + 1);
@@ -1232,6 +1188,7 @@ totem_pl_parser_add_smil (TotemPlParser *parser, const char *url, gpointer data)
 		if (parse_smil_entries (parser, base, doc, node) != FALSE)
 			retval = TOTEM_PL_PARSER_RESULT_SUCCESS;
 
+	g_free (base);
 	xmlFreeDoc (doc);
 
 	return retval;
@@ -1242,7 +1199,8 @@ totem_pl_parser_add_asf (TotemPlParser *parser, const char *url, gpointer data)
 {
 	if (data != NULL &&
 		(g_str_has_prefix (data, "[Reference]") == FALSE
-		 && g_ascii_strncasecmp (data, "<ASX", strlen ("<ASX")) != 0)) {
+		 && g_ascii_strncasecmp (data, "<ASX", strlen ("<ASX")) != 0
+		 && strstr (data, "<ASX") == NULL)) {
 		totem_pl_parser_add_one_url (parser, url, NULL);
 		return TOTEM_PL_PARSER_RESULT_SUCCESS;
 	}
@@ -1257,7 +1215,7 @@ totem_pl_parser_add_desktop (TotemPlParser *parser, const char *url, gpointer da
 	const char *path, *display_name, *type;
 	int size;
 
-	if (my_eel_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	contents = g_realloc (contents, size + 1);
@@ -1346,8 +1304,12 @@ totem_pl_parser_add_directory (TotemPlParser *parser, const char *url,
 			continue;
 		}
 
-		name = gnome_vfs_escape_string (info->name);
-		str = g_build_filename (url, name, NULL);
+		if (g_utf8_validate (info->name, -1, NULL) == FALSE)
+			name = g_strdup (info->name);
+		else
+			name = gnome_vfs_escape_string (info->name);
+		str = g_build_filename (url, info->name, NULL);
+
 		if (strstr (str, "://") != NULL && str[0] == '/')
 			fullpath = str + 1;
 		else
