@@ -1103,6 +1103,16 @@ on_recent_file_activate (EggRecentViewGtk *view, EggRecentItem *item,
 	g_free (uri);
 }
 
+static void
+on_got_redirect (BaconVideoWidget *bvw, const char *mrl, Totem *totem)
+{
+	g_message ("on_got_redirect %s", mrl);
+	//FIXME we need to check for relative paths here
+	bacon_video_widget_close (totem->bvw);
+	bacon_video_widget_open (totem->bvw, mrl, NULL);
+	bacon_video_widget_play (bvw, NULL);
+}
+
 /* This is only called when we are playing a DVD */
 static void
 on_title_change_event (BaconVideoWidget *bvw, const char *string, Totem *totem)
@@ -1182,6 +1192,8 @@ update_seekable (Totem *totem, gboolean seekable)
 	if (totem->seekable == seekable)
 		return;
 
+	totem->seekable = seekable;
+
 	/* Check if the stream is seekable */
 	gtk_widget_set_sensitive (totem->seek, seekable);
 	gtk_widget_set_sensitive (totem->fs_seek, seekable);
@@ -3026,7 +3038,7 @@ totem_callback_connect (Totem *totem)
 			G_CALLBACK (drop_playlist_cb), totem);
 	gtk_drag_dest_set (item, GTK_DEST_DEFAULT_ALL,
 			target_table, G_N_ELEMENTS (target_table),
-			GDK_ACTION_COPY);
+			GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 	/* Main Window */
 	g_signal_connect (G_OBJECT (totem->win), "delete-event",
@@ -3242,6 +3254,10 @@ video_widget_create (Totem *totem)
 			"eos",
 			G_CALLBACK (on_eos_event),
 			totem);
+	g_signal_connect (G_OBJECT (totem->bvw),
+			"got-redirect",
+			G_CALLBACK (on_got_redirect),
+			totem);
 	g_signal_connect (G_OBJECT(totem->bvw),
 			"title-change",
 			G_CALLBACK (on_title_change_event),
@@ -3284,7 +3300,7 @@ video_widget_create (Totem *totem)
 			G_CALLBACK (drop_video_cb), totem);
 	gtk_drag_dest_set (GTK_WIDGET (totem->bvw), GTK_DEST_DEFAULT_ALL,
 			target_table, G_N_ELEMENTS (target_table),
-			GDK_ACTION_COPY);
+			GDK_ACTION_COPY | GDK_ACTION_MOVE);
 
 	g_signal_connect (G_OBJECT (totem->bvw), "drag_data_get",
 			G_CALLBACK (drag_video_cb), totem);
